@@ -71,8 +71,8 @@ class MockMemcached(object):
     # In particular, the "Storage commands" section may be interesting.
 
     def __init__(self):
-        self.inbuf = ''
-        self.outbuf = ''
+        self.inbuf = b''
+        self.outbuf = b''
         self.cache = {}
         self.down = False
         self.exc_on_delete = False
@@ -84,81 +84,86 @@ class MockMemcached(object):
         if self.down:
             raise Exception('mock is down')
         self.inbuf += string
-        while '\n' in self.inbuf:
-            cmd, self.inbuf = self.inbuf.split('\n', 1)
+        while b'\n' in self.inbuf:
+            cmd, self.inbuf = self.inbuf.split(b'\n', 1)
             parts = cmd.split()
-            handler = getattr(self, 'handle_%s' % parts[0].lower(), None)
+            cmd_name = parts[0].decode('ascii').lower()
+            handler = getattr(self, 'handle_%s' % cmd_name, None)
             if handler:
                 handler(*parts[1:])
             else:
                 raise ValueError('Unhandled command: %s' % parts[0])
 
-    def handle_set(self, key, flags, exptime, num_bytes, noreply=''):
+    def handle_set(self, key, flags, exptime, num_bytes, noreply=b''):
         self.cache[key] = flags, exptime, self.inbuf[:int(num_bytes)]
         self.inbuf = self.inbuf[int(num_bytes) + 2:]
-        if noreply != 'noreply':
-            self.outbuf += 'STORED\r\n'
+        if noreply != b'noreply':
+            self.outbuf += b'STORED\r\n'
 
-    def handle_add(self, key, flags, exptime, num_bytes, noreply=''):
+    def handle_add(self, key, flags, exptime, num_bytes, noreply=b''):
         value = self.inbuf[:int(num_bytes)]
         self.inbuf = self.inbuf[int(num_bytes) + 2:]
         if key in self.cache:
-            if noreply != 'noreply':
-                self.outbuf += 'NOT_STORED\r\n'
+            if noreply != b'noreply':
+                self.outbuf += b'NOT_STORED\r\n'
         else:
             self.cache[key] = flags, exptime, value
-            if noreply != 'noreply':
-                self.outbuf += 'STORED\r\n'
+            if noreply != b'noreply':
+                self.outbuf += b'STORED\r\n'
 
-    def handle_delete(self, key, noreply=''):
+    def handle_delete(self, key, noreply=b''):
         if self.exc_on_delete:
             raise Exception('mock is has exc_on_delete set')
         if key in self.cache:
             del self.cache[key]
-            if noreply != 'noreply':
-                self.outbuf += 'DELETED\r\n'
-        elif noreply != 'noreply':
-            self.outbuf += 'NOT_FOUND\r\n'
+            if noreply != b'noreply':
+                self.outbuf += b'DELETED\r\n'
+        elif noreply != b'noreply':
+            self.outbuf += b'NOT_FOUND\r\n'
 
     def handle_get(self, *keys):
         for key in keys:
             if key in self.cache:
                 val = self.cache[key]
-                self.outbuf += 'VALUE %s %s %s\r\n' % (
-                    key, val[0], len(val[2]))
-                self.outbuf += val[2] + '\r\n'
-        self.outbuf += 'END\r\n'
+                self.outbuf += b' '.join([
+                    b'VALUE',
+                    key,
+                    val[0],
+                    str(len(val[2])).encode('ascii')
+                ]) + b'\r\n'
+                self.outbuf += val[2] + b'\r\n'
+        self.outbuf += b'END\r\n'
 
-    def handle_incr(self, key, value, noreply=''):
+    def handle_incr(self, key, value, noreply=b''):
         if key in self.cache:
             current = self.cache[key][2]
-            new_val = str(int(current) + int(value))
+            new_val = str(int(current) + int(value)).encode('ascii')
             self.cache[key] = self.cache[key][:2] + (new_val, )
-            self.outbuf += str(new_val) + '\r\n'
+            self.outbuf += new_val + b'\r\n'
         else:
-            self.outbuf += 'NOT_FOUND\r\n'
+            self.outbuf += b'NOT_FOUND\r\n'
 
-    def handle_decr(self, key, value, noreply=''):
+    def handle_decr(self, key, value, noreply=b''):
         if key in self.cache:
             current = self.cache[key][2]
-            new_val = str(int(current) - int(value))
-            if new_val[0] == '-':  # ie, val is negative
-                new_val = '0'
+            new_val = str(int(current) - int(value)).encode('ascii')
+            if new_val[:1] == b'-':  # ie, val is negative
+                new_val = b'0'
             self.cache[key] = self.cache[key][:2] + (new_val, )
-            self.outbuf += str(new_val) + '\r\n'
+            self.outbuf += new_val + b'\r\n'
         else:
-            self.outbuf += 'NOT_FOUND\r\n'
+            self.outbuf += b'NOT_FOUND\r\n'
 
     def readline(self):
         if self.read_return_empty_str:
-            return ''
+            return b''
         if self.read_return_none:
             return None
         if self.down:
             raise Exception('mock is down')
-        if '\n' in self.outbuf:
-            response, self.outbuf = self.outbuf.split('\n', 1)
-            return response + '\n'
+        if b'\n' in self.outbuf:
+            response, self.outbuf = self.outbuf.split(b'\n', 1)
+            return response + b'\n'
 
     def read(self, size):
         if self.down:
@@ -199,7 +204,7 @@ class TestMemcached(unittest.TestCase):
             memcache_client = memcached.MemcacheRing([sock1ipport, sock2ip])
             one = two = True
             while one or two:  # Run until we match hosts one and two
-                key = uuid4().hex
+                key = uuid4().hex.encode('ascii')
                 for conn in memcache_client._get_conns(key):
                     peeripport = '%s:%s' % conn[2].getpeername()
                     self.assertTrue(peeripport in (sock1ipport, sock2ipport))
@@ -222,7 +227,7 @@ class TestMemcached(unittest.TestCase):
             sock_addr = sock.getsockname()
             server_socket = '[%s]:%s' % (sock_addr[0], sock_addr[1])
             memcache_client = memcached.MemcacheRing([server_socket])
-            key = uuid4().hex
+            key = uuid4().hex.encode('ascii')
             for conn in memcache_client._get_conns(key):
                 peer_sockaddr = conn[2].getpeername()
                 peer_socket = '[%s]:%s' % (peer_sockaddr[0], peer_sockaddr[1])
@@ -243,7 +248,7 @@ class TestMemcached(unittest.TestCase):
             server_host = '[%s]' % sock_addr[0]
             memcached.DEFAULT_MEMCACHED_PORT = sock_addr[1]
             memcache_client = memcached.MemcacheRing([server_host])
-            key = uuid4().hex
+            key = uuid4().hex.encode('ascii')
             for conn in memcache_client._get_conns(key):
                 peer_sockaddr = conn[2].getpeername()
                 peer_socket = '[%s]:%s' % (peer_sockaddr[0], peer_sockaddr[1])
@@ -271,7 +276,7 @@ class TestMemcached(unittest.TestCase):
                                           socket.SOCK_STREAM, 0, '',
                                           ('127.0.0.1', sock_addr[1]))]
                 memcache_client = memcached.MemcacheRing([server_socket])
-                key = uuid4().hex
+                key = uuid4().hex.encode('ascii')
                 for conn in memcache_client._get_conns(key):
                     peer_sockaddr = conn[2].getpeername()
                     peer_socket = '%s:%s' % (peer_sockaddr[0],
@@ -296,7 +301,7 @@ class TestMemcached(unittest.TestCase):
                                           socket.SOCK_STREAM, 0, '',
                                           ('::1', sock_addr[1]))]
                 memcache_client = memcached.MemcacheRing([server_socket])
-                key = uuid4().hex
+                key = uuid4().hex.encode('ascii')
                 for conn in memcache_client._get_conns(key):
                     peer_sockaddr = conn[2].getpeername()
                     peer_socket = '[%s]:%s' % (peer_sockaddr[0],
@@ -312,16 +317,16 @@ class TestMemcached(unittest.TestCase):
         mock = MockMemcached()
         memcache_client._client_cache['1.2.3.4:11211'] = MockedMemcachePool(
             [(mock, mock)] * 2)
-        cache_key = md5('some_key').hexdigest()
+        cache_key = md5(b'some_key').hexdigest().encode('ascii')
 
         memcache_client.set('some_key', [1, 2, 3])
         self.assertEqual(memcache_client.get('some_key'), [1, 2, 3])
         # See JSON_FLAG
-        self.assertEqual(mock.cache, {cache_key: ('2', '0', '[1, 2, 3]')})
+        self.assertEqual(mock.cache, {cache_key: (b'2', b'0', b'[1, 2, 3]')})
 
         memcache_client.set('some_key', [4, 5, 6])
         self.assertEqual(memcache_client.get('some_key'), [4, 5, 6])
-        self.assertEqual(mock.cache, {cache_key: ('2', '0', '[4, 5, 6]')})
+        self.assertEqual(mock.cache, {cache_key: (b'2', b'0', b'[4, 5, 6]')})
 
         memcache_client.set('some_key', ['simple str', 'utf8 str éà'])
         # As per http://wiki.openstack.org/encoding,
@@ -329,10 +334,10 @@ class TestMemcached(unittest.TestCase):
         self.assertEqual(
             memcache_client.get('some_key'), ['simple str', u'utf8 str éà'])
         self.assertEqual(mock.cache, {cache_key: (
-            '2', '0', '["simple str", "utf8 str \\u00e9\\u00e0"]')})
+            b'2', b'0', b'["simple str", "utf8 str \\u00e9\\u00e0"]')})
 
         memcache_client.set('some_key', [1, 2, 3], time=20)
-        self.assertEqual(mock.cache, {cache_key: ('2', '20', '[1, 2, 3]')})
+        self.assertEqual(mock.cache, {cache_key: (b'2', b'20', b'[1, 2, 3]')})
 
         sixtydays = 60 * 24 * 60 * 60
         esttimeout = time.time() + sixtydays
@@ -347,7 +352,8 @@ class TestMemcached(unittest.TestCase):
             [(mock, mock)] * 2)
         memcache_client.set('some_key', [1, 2, 3])
         self.assertEqual(memcache_client.get('some_key'), [1, 2, 3])
-        self.assertEqual(mock.cache.values()[0][1], '0')
+        self.assertEqual(list(mock.cache.values()),
+                         [(b'2', b'0', b'[1, 2, 3]')])
 
         # Now lets return an empty string, and make sure we aren't logging
         # the error.
@@ -359,7 +365,7 @@ class TestMemcached(unittest.TestCase):
         with patch("sys.stdout", fake_stdout),\
                 patch('swift.common.memcached.logging', logger):
             mock.read_return_empty_str = True
-            self.assertEqual(memcache_client.get('some_key'), None)
+            self.assertIsNone(memcache_client.get('some_key'))
         log_lines = logger.get_lines_for_level('error')
         self.assertIn('Error talking to memcached', log_lines[0])
         self.assertFalse(log_lines[1:])
@@ -371,15 +377,15 @@ class TestMemcached(unittest.TestCase):
         memcache_client._client_cache['1.2.3.4:11211'] = MockedMemcachePool(
             [(mock, mock)] * 2)
         self.assertEqual(memcache_client.incr('some_key', delta=5), 5)
-        self.assertEqual(memcache_client.get('some_key'), '5')
+        self.assertEqual(memcache_client.get('some_key'), b'5')
         self.assertEqual(memcache_client.incr('some_key', delta=5), 10)
-        self.assertEqual(memcache_client.get('some_key'), '10')
+        self.assertEqual(memcache_client.get('some_key'), b'10')
         self.assertEqual(memcache_client.incr('some_key', delta=1), 11)
-        self.assertEqual(memcache_client.get('some_key'), '11')
+        self.assertEqual(memcache_client.get('some_key'), b'11')
         self.assertEqual(memcache_client.incr('some_key', delta=-5), 6)
-        self.assertEqual(memcache_client.get('some_key'), '6')
+        self.assertEqual(memcache_client.get('some_key'), b'6')
         self.assertEqual(memcache_client.incr('some_key', delta=-15), 0)
-        self.assertEqual(memcache_client.get('some_key'), '0')
+        self.assertEqual(memcache_client.get('some_key'), b'0')
         mock.read_return_none = True
         self.assertRaises(memcached.MemcacheConnectionError,
                           memcache_client.incr, 'some_key', delta=-15)
@@ -391,9 +397,9 @@ class TestMemcached(unittest.TestCase):
         memcache_client._client_cache['1.2.3.4:11211'] = MockedMemcachePool(
             [(mock, mock)] * 2)
         self.assertEqual(memcache_client.incr('some_key', delta=5), 5)
-        self.assertEqual(memcache_client.get('some_key'), '5')
+        self.assertEqual(memcache_client.get('some_key'), b'5')
         self.assertEqual(memcache_client.incr('some_key', delta=5), 10)
-        self.assertEqual(memcache_client.get('some_key'), '10')
+        self.assertEqual(memcache_client.get('some_key'), b'10')
 
         # Now lets return an empty string, and make sure we aren't logging
         # the error.
@@ -417,11 +423,11 @@ class TestMemcached(unittest.TestCase):
         mock = MockMemcached()
         memcache_client._client_cache['1.2.3.4:11211'] = MockedMemcachePool(
             [(mock, mock)] * 2)
-        cache_key = md5('some_key').hexdigest()
+        cache_key = md5(b'some_key').hexdigest().encode('ascii')
 
         memcache_client.incr('some_key', delta=5, time=55)
-        self.assertEqual(memcache_client.get('some_key'), '5')
-        self.assertEqual(mock.cache, {cache_key: ('0', '55', '5')})
+        self.assertEqual(memcache_client.get('some_key'), b'5')
+        self.assertEqual(mock.cache, {cache_key: (b'0', b'55', b'5')})
 
         memcache_client.delete('some_key')
         self.assertIsNone(memcache_client.get('some_key'))
@@ -429,7 +435,7 @@ class TestMemcached(unittest.TestCase):
         fiftydays = 50 * 24 * 60 * 60
         esttimeout = time.time() + fiftydays
         memcache_client.incr('some_key', delta=5, time=fiftydays)
-        self.assertEqual(memcache_client.get('some_key'), '5')
+        self.assertEqual(memcache_client.get('some_key'), b'5')
         _junk, cache_timeout, _junk = mock.cache[cache_key]
         self.assertAlmostEqual(float(cache_timeout), esttimeout, delta=1)
 
@@ -437,12 +443,12 @@ class TestMemcached(unittest.TestCase):
         self.assertIsNone(memcache_client.get('some_key'))
 
         memcache_client.incr('some_key', delta=5)
-        self.assertEqual(memcache_client.get('some_key'), '5')
-        self.assertEqual(mock.cache, {cache_key: ('0', '0', '5')})
+        self.assertEqual(memcache_client.get('some_key'), b'5')
+        self.assertEqual(mock.cache, {cache_key: (b'0', b'0', b'5')})
 
         memcache_client.incr('some_key', delta=5, time=55)
-        self.assertEqual(memcache_client.get('some_key'), '10')
-        self.assertEqual(mock.cache, {cache_key: ('0', '0', '10')})
+        self.assertEqual(memcache_client.get('some_key'), b'10')
+        self.assertEqual(mock.cache, {cache_key: (b'0', b'0', b'10')})
 
     def test_decr(self):
         memcache_client = memcached.MemcacheRing(['1.2.3.4:11211'])
@@ -450,13 +456,13 @@ class TestMemcached(unittest.TestCase):
         memcache_client._client_cache['1.2.3.4:11211'] = MockedMemcachePool(
             [(mock, mock)] * 2)
         self.assertEqual(memcache_client.decr('some_key', delta=5), 0)
-        self.assertEqual(memcache_client.get('some_key'), '0')
+        self.assertEqual(memcache_client.get('some_key'), b'0')
         self.assertEqual(memcache_client.incr('some_key', delta=15), 15)
-        self.assertEqual(memcache_client.get('some_key'), '15')
+        self.assertEqual(memcache_client.get('some_key'), b'15')
         self.assertEqual(memcache_client.decr('some_key', delta=4), 11)
-        self.assertEqual(memcache_client.get('some_key'), '11')
+        self.assertEqual(memcache_client.get('some_key'), b'11')
         self.assertEqual(memcache_client.decr('some_key', delta=15), 0)
-        self.assertEqual(memcache_client.get('some_key'), '0')
+        self.assertEqual(memcache_client.get('some_key'), b'0')
         mock.read_return_none = True
         self.assertRaises(memcached.MemcacheConnectionError,
                           memcache_client.decr, 'some_key', delta=15)
@@ -510,27 +516,27 @@ class TestMemcached(unittest.TestCase):
         self.assertEqual(
             memcache_client.get_multi(('some_key2', 'some_key1'), 'multi_key'),
             [[4, 5, 6], [1, 2, 3]])
-        for key in ('some_key1', 'some_key2'):
-            key = md5(key).hexdigest()
+        for key in (b'some_key1', b'some_key2'):
+            key = md5(key).hexdigest().encode('ascii')
             self.assertIn(key, mock.cache)
             _junk, cache_timeout, _junk = mock.cache[key]
-            self.assertEqual(cache_timeout, '0')
+            self.assertEqual(cache_timeout, b'0')
 
         memcache_client.set_multi(
             {'some_key1': [1, 2, 3], 'some_key2': [4, 5, 6]}, 'multi_key',
             time=20)
-        for key in ('some_key1', 'some_key2'):
-            key = md5(key).hexdigest()
+        for key in (b'some_key1', b'some_key2'):
+            key = md5(key).hexdigest().encode('ascii')
             _junk, cache_timeout, _junk = mock.cache[key]
-            self.assertEqual(cache_timeout, '20')
+            self.assertEqual(cache_timeout, b'20')
 
         fortydays = 50 * 24 * 60 * 60
         esttimeout = time.time() + fortydays
         memcache_client.set_multi(
             {'some_key1': [1, 2, 3], 'some_key2': [4, 5, 6]}, 'multi_key',
             time=fortydays)
-        for key in ('some_key1', 'some_key2'):
-            key = md5(key).hexdigest()
+        for key in (b'some_key1', b'some_key2'):
+            key = md5(key).hexdigest().encode('ascii')
             _junk, cache_timeout, _junk = mock.cache[key]
             self.assertAlmostEqual(float(cache_timeout), esttimeout, delta=1)
         self.assertEqual(memcache_client.get_multi(
