@@ -16,16 +16,17 @@
 from time import time
 from unittest import main, TestCase
 from test.unit import FakeRing, mocked_http_conn, debug_logger
-from copy import deepcopy
 from tempfile import mkdtemp
 from shutil import rmtree
 from collections import defaultdict
+from copy import deepcopy
 
 import mock
 import six
 from six.moves import urllib
 
 from swift.common import internal_client, utils, swob
+from swift.common.utils import Timestamp
 from swift.obj import expirer
 
 
@@ -61,12 +62,15 @@ class FakeInternalClient(object):
         self.aco_dict.update(aco_dict)
 
     def get_account_info(self, account):
-        return 1, 2
+        acc_dict = self.aco_dict[account]
+        container_count = len(acc_dict)
+        obj_count = sum(len(objs) for objs in acc_dict.values())
+        return container_count, obj_count
 
     def iter_containers(self, account, prefix=''):
         acc_dict = self.aco_dict[account]
-        return [{'name': six.text_type(container)} for container in
-                acc_dict if container.startswith(prefix)]
+        return sorted([{'name': six.text_type(container)} for container in
+                       acc_dict if container.startswith(prefix)])
 
     def delete_container(*a, **kw):
         pass
@@ -97,6 +101,41 @@ class TestObjectExpirer(TestCase):
         self.conf = {'recon_cache_path': self.rcache}
         self.logger = debug_logger('test-expirer')
 
+        self.past_time = str(int(time() - 86400))
+        self.future_time = str(int(time() + 86400))
+        # Dummy task queue for test
+        self.fake_swift = FakeInternalClient({
+            '.expiring_objects': {
+                # this task container will be checked
+                self.past_time: [
+                    # tasks ready for execution
+                    self.past_time + '-a0/c0/o0',
+                    self.past_time + '-a1/c1/o1',
+                    self.past_time + '-a2/c2/o2',
+                    self.past_time + '-a3/c3/o3',
+                    self.past_time + '-a4/c4/o4',
+                    self.past_time + '-a5/c5/o5',
+                    self.past_time + '-a6/c6/o6',
+                    self.past_time + '-a7/c7/o7',
+                    # task objects for unicode test
+                    self.past_time + u'-a8/c8/o8\u2661',
+                    self.past_time + u'-a9/c9/o9\xf8',
+                    # this task will be skipped
+                    self.future_time + '-a10/c10/o10'],
+                # this task container will be skipped
+                self.future_time: [
+                    self.future_time + '-a11/c11/o11']}
+        })
+        self.expirer = expirer.ObjectExpirer(self.conf, logger=self.logger,
+                                             swift=self.fake_swift)
+
+        # target object paths which should be expirerd now
+        self.expired_target_path_list = [
+            'a0/c0/o0', 'a1/c1/o1', 'a2/c2/o2', 'a3/c3/o3', 'a4/c4/o4',
+            'a5/c5/o5', 'a6/c6/o6', 'a7/c7/o7',
+            'a8/c8/o8\xe2\x99\xa1', 'a9/c9/o9\xc3\xb8',
+        ]
+
     def tearDown(self):
         rmtree(self.rcache)
         internal_client.sleep = self.old_sleep
@@ -210,41 +249,40 @@ class TestObjectExpirer(TestCase):
                 super(ObjectExpirer, self).__init__(conf, swift=swift)
                 self.processes = 3
                 self.deleted_objects = {}
-                self.obj_containers_in_order = []
 
-            def delete_object(self, actual_obj, timestamp, container, obj):
-                if container not in self.deleted_objects:
-                    self.deleted_objects[container] = set()
-                self.deleted_objects[container].add(obj)
-                self.obj_containers_in_order.append(container)
+            def delete_object(self, target_path, delete_timestamp,
+                              task_account, task_container, task_object):
+                if task_container not in self.deleted_objects:
+                    self.deleted_objects[task_container] = set()
+                self.deleted_objects[task_container].add(task_object)
 
-        aco_dict = {
-            '.expiring_objects': {
-                '0': set('1-one 2-two 3-three'.split()),
-                '1': set('2-two 3-three 4-four'.split()),
-                '2': set('5-five 6-six'.split()),
-                '3': set(u'7-seven\u2661'.split()),
-            },
-        }
-        fake_swift = FakeInternalClient(aco_dict)
-        x = ObjectExpirer(self.conf, swift=fake_swift)
+        x = ObjectExpirer(self.conf, swift=self.fake_swift)
 
-        deleted_objects = {}
+        deleted_objects = defaultdict(set)
         for i in range(3):
             x.process = i
+            # reset progress so we know we don't double-up work among processes
+            x.deleted_objects = defaultdict(set)
             x.run_once()
-            self.assertNotEqual(deleted_objects, x.deleted_objects)
-            deleted_objects = deepcopy(x.deleted_objects)
-        self.assertEqual(aco_dict['.expiring_objects']['3'].pop(),
-                         deleted_objects['3'].pop().decode('utf8'))
-        self.assertEqual(aco_dict['.expiring_objects'], deleted_objects)
-        self.assertEqual(len(set(x.obj_containers_in_order[:4])), 4)
+            for task_container, deleted in x.deleted_objects.items():
+                self.assertFalse(deleted_objects[task_container] & deleted)
+                deleted_objects[task_container] |= deleted
+
+        # sort for comparison
+        deleted_objects = {
+            con: sorted(o_set) for con, o_set in deleted_objects.items()}
+        expected = {
+            self.past_time: [
+                self.past_time + '-' + target_path
+                for target_path in self.expired_target_path_list]}
+        self.assertEqual(deleted_objects, expected)
 
     def test_delete_object(self):
         x = expirer.ObjectExpirer({}, logger=self.logger)
         actual_obj = 'actual_obj'
         timestamp = int(time())
         reclaim_ts = timestamp - x.reclaim_age
+        account = 'account'
         container = 'container'
         obj = 'obj'
 
@@ -262,12 +300,12 @@ class TestObjectExpirer(TestCase):
             with mock.patch.object(x, 'delete_actual_object',
                                    side_effect=exc) as delete_actual:
                 with mock.patch.object(x, 'pop_queue') as pop_queue:
-                    x.delete_object(actual_obj, ts, container, obj)
+                    x.delete_object(actual_obj, ts, account, container, obj)
 
             delete_actual.assert_called_once_with(actual_obj, ts)
             log_lines = x.logger.get_lines_for_level('error')
             if should_pop:
-                pop_queue.assert_called_once_with(container, obj)
+                pop_queue.assert_called_once_with(account, container, obj)
                 self.assertEqual(start_reports + 1, x.report_objects)
                 self.assertFalse(log_lines)
             else:
@@ -277,11 +315,12 @@ class TestObjectExpirer(TestCase):
                 if isinstance(exc, internal_client.UnexpectedResponse):
                     self.assertEqual(
                         log_lines[0],
-                        'Unexpected response while deleting object container '
-                        'obj: %s' % exc.resp.status_int)
+                        'Unexpected response while deleting object '
+                        'account container obj: %s' % exc.resp.status_int)
                 else:
                     self.assertTrue(log_lines[0].startswith(
-                        'Exception while deleting object container obj'))
+                        'Exception while deleting object '
+                        'account container obj'))
 
         # verify pop_queue logic on exceptions
         for exc, ts, should_pop in [(None, timestamp, True),
@@ -318,6 +357,171 @@ class TestObjectExpirer(TestCase):
         self.assertTrue(
             'so far' in str(x.logger.get_lines_for_level('info')))
 
+    def test_parse_task_obj(self):
+        x = expirer.ObjectExpirer(self.conf, logger=self.logger)
+
+        def assert_parse_task_obj(task_obj, expected_delete_at,
+                                  expected_account, expected_container,
+                                  expected_obj):
+            delete_at, account, container, obj = x.parse_task_obj(task_obj)
+            self.assertEqual(delete_at, expected_delete_at)
+            self.assertEqual(account, expected_account)
+            self.assertEqual(container, expected_container)
+            self.assertEqual(obj, expected_obj)
+
+        assert_parse_task_obj('0000-a/c/o', 0, 'a', 'c', 'o')
+        assert_parse_task_obj('0001-a/c/o', 1, 'a', 'c', 'o')
+        assert_parse_task_obj('1000-a/c/o', 1000, 'a', 'c', 'o')
+        assert_parse_task_obj('0000-acc/con/obj', 0, 'acc', 'con', 'obj')
+
+    def make_task(self, delete_at, target):
+        return {
+            'task_account': '.expiring_objects',
+            'task_container': delete_at,
+            'task_object': delete_at + '-' + target,
+            'delete_timestamp': Timestamp(delete_at),
+            'target_path': target,
+        }
+
+    def test_round_robin_order(self):
+        x = expirer.ObjectExpirer(self.conf, logger=self.logger)
+        task_con_obj_list = [
+            # objects in 0000 timestamp container
+            self.make_task('0000', 'a/c0/o0'),
+            self.make_task('0000', 'a/c0/o1'),
+            # objects in 0001 timestamp container
+            self.make_task('0001', 'a/c1/o0'),
+            self.make_task('0001', 'a/c1/o1'),
+            # objects in 0002 timestamp container
+            self.make_task('0002', 'a/c2/o0'),
+            self.make_task('0002', 'a/c2/o1'),
+        ]
+        result = list(x.round_robin_order(task_con_obj_list))
+
+        # sorted by popping one object to delete for each target_container
+        expected = [
+            self.make_task('0000', 'a/c0/o0'),
+            self.make_task('0001', 'a/c1/o0'),
+            self.make_task('0002', 'a/c2/o0'),
+            self.make_task('0000', 'a/c0/o1'),
+            self.make_task('0001', 'a/c1/o1'),
+            self.make_task('0002', 'a/c2/o1'),
+        ]
+        self.assertEqual(expected, result)
+
+        # task containers have some task objects with invalid target paths
+        task_con_obj_list = [
+            # objects in 0000 timestamp container
+            self.make_task('0000', 'invalid0'),
+            self.make_task('0000', 'a/c0/o0'),
+            self.make_task('0000', 'a/c0/o1'),
+            # objects in 0001 timestamp container
+            self.make_task('0001', 'a/c1/o0'),
+            self.make_task('0001', 'invalid1'),
+            self.make_task('0001', 'a/c1/o1'),
+            # objects in 0002 timestamp container
+            self.make_task('0002', 'a/c2/o0'),
+            self.make_task('0002', 'a/c2/o1'),
+            self.make_task('0002', 'invalid2'),
+        ]
+        result = list(x.round_robin_order(task_con_obj_list))
+
+        # the invalid task objects are ignored
+        expected = [
+            self.make_task('0000', 'a/c0/o0'),
+            self.make_task('0001', 'a/c1/o0'),
+            self.make_task('0002', 'a/c2/o0'),
+            self.make_task('0000', 'a/c0/o1'),
+            self.make_task('0001', 'a/c1/o1'),
+            self.make_task('0002', 'a/c2/o1'),
+        ]
+        self.assertEqual(expected, result)
+
+        # for a given target container, tasks won't necessarily all go in
+        # the same timestamp container
+        task_con_obj_list = [
+            # objects in 0000 timestamp container
+            self.make_task('0000', 'a/c0/o0'),
+            self.make_task('0000', 'a/c0/o1'),
+            self.make_task('0000', 'a/c2/o2'),
+            self.make_task('0000', 'a/c2/o3'),
+            # objects in 0001 timestamp container
+            self.make_task('0001', 'a/c0/o2'),
+            self.make_task('0001', 'a/c0/o3'),
+            self.make_task('0001', 'a/c1/o0'),
+            self.make_task('0001', 'a/c1/o1'),
+            # objects in 0002 timestamp container
+            self.make_task('0002', 'a/c2/o0'),
+            self.make_task('0002', 'a/c2/o1'),
+        ]
+        result = list(x.round_robin_order(task_con_obj_list))
+
+        # so we go around popping by *target* container, not *task* container
+        expected = [
+            self.make_task('0000', 'a/c0/o0'),
+            self.make_task('0001', 'a/c1/o0'),
+            self.make_task('0000', 'a/c2/o2'),
+            self.make_task('0000', 'a/c0/o1'),
+            self.make_task('0001', 'a/c1/o1'),
+            self.make_task('0000', 'a/c2/o3'),
+            self.make_task('0001', 'a/c0/o2'),
+            self.make_task('0002', 'a/c2/o0'),
+            self.make_task('0001', 'a/c0/o3'),
+            self.make_task('0002', 'a/c2/o1'),
+        ]
+        self.assertEqual(expected, result)
+
+        # all of the work to be done could be for different target containers
+        task_con_obj_list = [
+            # objects in 0000 timestamp container
+            self.make_task('0000', 'a/c0/o'),
+            self.make_task('0000', 'a/c1/o'),
+            self.make_task('0000', 'a/c2/o'),
+            self.make_task('0000', 'a/c3/o'),
+            # objects in 0001 timestamp container
+            self.make_task('0001', 'a/c4/o'),
+            self.make_task('0001', 'a/c5/o'),
+            self.make_task('0001', 'a/c6/o'),
+            self.make_task('0001', 'a/c7/o'),
+            # objects in 0002 timestamp container
+            self.make_task('0002', 'a/c8/o'),
+            self.make_task('0002', 'a/c9/o'),
+        ]
+        result = list(x.round_robin_order(task_con_obj_list))
+
+        # in which case, we kind of hammer the task containers
+        self.assertEqual(task_con_obj_list, result)
+
+    def test_hash_mod(self):
+        x = expirer.ObjectExpirer(self.conf, logger=self.logger)
+        mod_count = [0, 0, 0]
+        for i in range(1000):
+            name = 'obj%d' % i
+            mod = x.hash_mod(name, 3)
+            mod_count[mod] += 1
+
+        # 1000 names are well shuffled
+        self.assertGreater(mod_count[0], 300)
+        self.assertGreater(mod_count[1], 300)
+        self.assertGreater(mod_count[2], 300)
+
+    def test_iter_task_accounts_to_expire(self):
+        x = expirer.ObjectExpirer(self.conf, logger=self.logger)
+        results = [_ for _ in x.iter_task_accounts_to_expire()]
+        self.assertEqual(results, [('.expiring_objects', 0, 1)])
+
+        self.conf['processes'] = '2'
+        self.conf['process'] = '1'
+        x = expirer.ObjectExpirer(self.conf, logger=self.logger)
+        results = [_ for _ in x.iter_task_accounts_to_expire()]
+        self.assertEqual(results, [('.expiring_objects', 1, 2)])
+
+    def test_delete_at_time_of_task_container(self):
+        x = expirer.ObjectExpirer(self.conf, logger=self.logger)
+        self.assertEqual(x.delete_at_time_of_task_container('0000'), 0)
+        self.assertEqual(x.delete_at_time_of_task_container('0001'), 1)
+        self.assertEqual(x.delete_at_time_of_task_container('1000'), 1000)
+
     def test_run_once_nothing_to_do(self):
         x = expirer.ObjectExpirer(self.conf, logger=self.logger)
         x.swift = 'throw error because a string does not have needed methods'
@@ -329,165 +533,128 @@ class TestObjectExpirer(TestCase):
                          "'str' object has no attribute 'get_account_info'")
 
     def test_run_once_calls_report(self):
-        fake_swift = FakeInternalClient({})
+        with mock.patch.object(self.expirer, 'pop_queue',
+                               lambda a, c, o: None):
+            self.expirer.run_once()
+        self.assertEqual(
+            self.expirer.logger.get_lines_for_level('info'), [
+                'Pass beginning for task account .expiring_objects; '
+                '2 possible containers; 12 possible objects',
+                'Pass completed in 0s; 10 objects expired',
+            ])
+
+    def test_skip_task_account_without_task_container(self):
+        fake_swift = FakeInternalClient({
+            # task account has no containers
+            '.expiring_objects': dict()
+        })
         x = expirer.ObjectExpirer(self.conf, logger=self.logger,
                                   swift=fake_swift)
         x.run_once()
         self.assertEqual(
             x.logger.get_lines_for_level('info'), [
-                'Pass beginning; 1 possible containers; 2 possible objects',
                 'Pass completed in 0s; 0 objects expired',
             ])
 
-    def test_run_once_unicode_problem(self):
-        fake_swift = FakeInternalClient({
-            '.expiring_objects': {u'1234': [u'1234-troms\xf8']}
-        })
-        x = expirer.ObjectExpirer(self.conf, logger=self.logger,
-                                  swift=fake_swift)
+    def test_iter_task_to_expire(self):
+        # In this test, all tasks are assigned to the tested expirer
+        my_index = 0
+        divisor = 1
 
+        task_account_container_list = [('.expiring_objects', self.past_time)]
+
+        expected = [
+            self.make_task(self.past_time, target_path)
+            for target_path in self.expired_target_path_list]
+
+        self.assertEqual(
+            list(self.expirer.iter_task_to_expire(
+                task_account_container_list, my_index, divisor)),
+            expected)
+
+        # the task queue has invalid task object
+        invalid_aco_dict = deepcopy(self.fake_swift.aco_dict)
+        invalid_aco_dict['.expiring_objects'][self.past_time].insert(
+            0, self.past_time + '-invalid0')
+        invalid_aco_dict['.expiring_objects'][self.past_time].insert(
+            5, self.past_time + '-invalid1')
+        invalid_fake_swift = FakeInternalClient(invalid_aco_dict)
+        x = expirer.ObjectExpirer(self.conf, logger=self.logger,
+                                  swift=invalid_fake_swift)
+
+        # but the invalid tasks are skipped
+        self.assertEqual(
+            list(x.iter_task_to_expire(
+                task_account_container_list, my_index, divisor)),
+            expected)
+
+    def test_run_once_unicode_problem(self):
         requests = []
 
         def capture_requests(ipaddr, port, method, path, *args, **kwargs):
             requests.append((method, path))
 
-        with mocked_http_conn(200, 200, 200, give_connect=capture_requests):
-            x.run_once()
-        self.assertEqual(len(requests), 3)
+        # 3 DELETE requests for each 10 executed task objects to pop_queue
+        code_list = [200] * 3 * 10
+        with mocked_http_conn(*code_list, give_connect=capture_requests):
+            self.expirer.run_once()
+        self.assertEqual(len(requests), 30)
 
     def test_container_timestamp_break(self):
-        def fail_to_iter_objects(*a, **kw):
-            raise Exception('This should not have been called')
+        with mock.patch.object(self.fake_swift, 'iter_objects') as mock_method:
+            self.expirer.run_once()
 
-        fake_swift = FakeInternalClient({
-            '.expiring_objects': {str(int(time() + 86400)): []}
-        })
-        x = expirer.ObjectExpirer(self.conf, logger=self.logger,
-                                  swift=fake_swift)
-        with mock.patch.object(fake_swift, 'iter_objects',
-                               fail_to_iter_objects):
-            x.run_once()
-        logs = x.logger.all_log_lines()
-        self.assertEqual(logs['info'], [
-            'Pass beginning; 1 possible containers; 2 possible objects',
-            'Pass completed in 0s; 0 objects expired',
-        ])
-        self.assertNotIn('error', logs)
-
-        # Reverse test to be sure it still would blow up the way expected.
-        fake_swift = FakeInternalClient({
-            '.expiring_objects': {str(int(time() - 86400)): []}
-        })
-        x = expirer.ObjectExpirer(self.conf, logger=self.logger,
-                                  swift=fake_swift)
-        with mock.patch.object(fake_swift, 'iter_objects',
-                               fail_to_iter_objects):
-            x.run_once()
-        self.assertEqual(
-            x.logger.get_lines_for_level('error'), ['Unhandled exception: '])
-        log_args, log_kwargs = x.logger.log_dict['error'][-1]
-        self.assertEqual(str(log_kwargs['exc_info'][1]),
-                         'This should not have been called')
+        # iter_objects is called only for past_time, not future_time
+        self.assertEqual(mock_method.call_args_list,
+                         [mock.call('.expiring_objects', self.past_time)])
 
     def test_object_timestamp_break(self):
-        def should_not_be_called(*a, **kw):
-            raise Exception('This should not have been called')
+        with mock.patch.object(self.expirer, 'delete_actual_object') \
+                as mock_method, \
+                mock.patch.object(self.expirer, 'pop_queue'):
+            self.expirer.run_once()
 
-        fake_swift = FakeInternalClient({
-            '.expiring_objects': {
-                str(int(time() - 86400)): [
-                    '%d-actual-obj' % int(time() + 86400)],
-            },
-        })
-        x = expirer.ObjectExpirer(self.conf, logger=self.logger,
-                                  swift=fake_swift)
-        x.run_once()
-        self.assertNotIn('error', x.logger.all_log_lines())
-        self.assertEqual(x.logger.get_lines_for_level('info'), [
-            'Pass beginning; 1 possible containers; 2 possible objects',
-            'Pass completed in 0s; 0 objects expired',
-        ])
-        # Reverse test to be sure it still would blow up the way expected.
-        ts = int(time() - 86400)
-        fake_swift = FakeInternalClient({
-            '.expiring_objects': {
-                str(int(time() - 86400)): ['%d-actual-obj' % ts],
-            },
-        })
-        x = expirer.ObjectExpirer(self.conf, logger=self.logger,
-                                  swift=fake_swift)
-        x.delete_actual_object = should_not_be_called
-        x.run_once()
+        # executed tasks are with past time
         self.assertEqual(
-            x.logger.get_lines_for_level('error'),
-            ['Exception while deleting object %d %d-actual-obj '
-             'This should not have been called: ' % (ts, ts)])
+            mock_method.call_args_list,
+            [mock.call(target_path, self.past_time)
+             for target_path in self.expired_target_path_list])
 
     def test_failed_delete_keeps_entry(self):
         def deliberately_blow_up(actual_obj, timestamp):
             raise Exception('failed to delete actual object')
 
-        def should_not_get_called(container, obj):
-            raise Exception('This should not have been called')
+        # any tasks are not done
+        with mock.patch.object(self.expirer, 'delete_actual_object',
+                               deliberately_blow_up), \
+                mock.patch.object(self.expirer, 'pop_queue') as mock_method:
+            self.expirer.run_once()
 
-        ts = int(time() - 86400)
-        fake_swift = FakeInternalClient({
-            '.expiring_objects': {
-                str(int(time() - 86400)): ['%d-actual-obj' % ts],
-            },
-        })
-        x = expirer.ObjectExpirer(self.conf, logger=self.logger,
-                                  swift=fake_swift)
-        x.delete_actual_object = deliberately_blow_up
-        x.pop_queue = should_not_get_called
-        x.run_once()
-        self.assertEqual(
-            x.logger.get_lines_for_level('error'),
-            ['Exception while deleting object %d %d-actual-obj '
-             'failed to delete actual object: ' % (ts, ts)])
-        self.assertEqual(
-            x.logger.get_lines_for_level('info'), [
-                'Pass beginning; 1 possible containers; 2 possible objects',
-                'Pass completed in 0s; 0 objects expired',
-            ])
+        # no tasks are popped from the queue
+        self.assertEqual(mock_method.call_args_list, [])
 
-        # Reverse test to be sure it still would blow up the way expected.
-        ts = int(time() - 86400)
-        fake_swift = FakeInternalClient({
-            '.expiring_objects': {
-                str(int(time() - 86400)): ['%d-actual-obj' % ts],
-            },
-        })
-        self.logger._clear()
-        x = expirer.ObjectExpirer(self.conf, logger=self.logger,
-                                  swift=fake_swift)
-        x.delete_actual_object = lambda o, t: None
-        x.pop_queue = should_not_get_called
-        x.run_once()
+        # all tasks are done
+        with mock.patch.object(self.expirer, 'delete_actual_object',
+                               lambda o, t: None), \
+                mock.patch.object(self.expirer, 'pop_queue') as mock_method:
+            self.expirer.run_once()
+
+        # all tasks are popped from the queue
         self.assertEqual(
-            self.logger.get_lines_for_level('error'),
-            ['Exception while deleting object %d %d-actual-obj This should '
-             'not have been called: ' % (ts, ts)])
+            mock_method.call_args_list,
+            [mock.call('.expiring_objects', self.past_time,
+             self.past_time + '-' + target_path)
+             for target_path in self.expired_target_path_list])
 
     def test_success_gets_counted(self):
-        fake_swift = FakeInternalClient({
-            '.expiring_objects': {
-                str(int(time() - 86400)): [
-                    '%d-acc/c/actual-obj' % int(time() - 86400)],
-            },
-        })
-        x = expirer.ObjectExpirer(self.conf, logger=self.logger,
-                                  swift=fake_swift)
-        x.delete_actual_object = lambda o, t: None
-        x.pop_queue = lambda c, o: None
-        self.assertEqual(x.report_objects, 0)
-        with mock.patch('swift.obj.expirer.MAX_OBJECTS_TO_CACHE', 0):
-            x.run_once()
-            self.assertEqual(x.report_objects, 1)
-            self.assertEqual(
-                x.logger.get_lines_for_level('info'),
-                ['Pass beginning; 1 possible containers; 2 possible objects',
-                 'Pass completed in 0s; 1 objects expired'])
+        self.assertEqual(self.expirer.report_objects, 0)
+        with mock.patch('swift.obj.expirer.MAX_OBJECTS_TO_CACHE', 0), \
+                mock.patch.object(self.expirer, 'delete_actual_object',
+                                  lambda o, t: None), \
+                mock.patch.object(self.expirer, 'pop_queue',
+                                  lambda a, c, o: None):
+            self.expirer.run_once()
+        self.assertEqual(self.expirer.report_objects, 10)
 
     def test_delete_actual_object_does_not_get_unicode(self):
         got_unicode = [False]
@@ -496,24 +663,15 @@ class TestObjectExpirer(TestCase):
             if isinstance(actual_obj, six.text_type):
                 got_unicode[0] = True
 
-        fake_swift = FakeInternalClient({
-            '.expiring_objects': {
-                str(int(time() - 86400)): [
-                    '%d-actual-obj' % int(time() - 86400)],
-            },
-        })
-        x = expirer.ObjectExpirer(self.conf, logger=self.logger,
-                                  swift=fake_swift)
-        x.delete_actual_object = delete_actual_object_test_for_unicode
-        x.pop_queue = lambda c, o: None
-        self.assertEqual(x.report_objects, 0)
-        x.run_once()
-        self.assertEqual(x.report_objects, 1)
-        self.assertEqual(
-            x.logger.get_lines_for_level('info'), [
-                'Pass beginning; 1 possible containers; 2 possible objects',
-                'Pass completed in 0s; 1 objects expired',
-            ])
+        self.assertEqual(self.expirer.report_objects, 0)
+
+        with mock.patch.object(self.expirer, 'delete_actual_object',
+                               delete_actual_object_test_for_unicode), \
+                mock.patch.object(self.expirer, 'pop_queue',
+                                  lambda a, c, o: None):
+            self.expirer.run_once()
+
+        self.assertEqual(self.expirer.report_objects, 10)
         self.assertFalse(got_unicode[0])
 
     def test_failed_delete_continues_on(self):
@@ -523,37 +681,26 @@ class TestObjectExpirer(TestCase):
         def fail_delete_actual_object(actual_obj, timestamp):
             raise Exception('failed to delete actual object')
 
-        cts = int(time() - 86400)
-        ots = int(time() - 86400)
+        with mock.patch.object(self.fake_swift, 'delete_container',
+                               fail_delete_container), \
+                mock.patch.object(self.expirer, 'delete_actual_object',
+                                  fail_delete_actual_object):
+            self.expirer.run_once()
 
-        fake_swift = FakeInternalClient({
-            '.expiring_objects': {
-                str(cts): ['%d-actual-obj' % ots, '%d-next-obj' % ots],
-                str(cts + 1): ['%d-actual-obj' % ots, '%d-next-obj' % ots],
-            },
-        })
-        x = expirer.ObjectExpirer(self.conf, logger=self.logger,
-                                  swift=fake_swift)
-        x.delete_actual_object = fail_delete_actual_object
-        with mock.patch.object(fake_swift, 'delete_container',
-                               fail_delete_container):
-            x.run_once()
-        error_lines = x.logger.get_lines_for_level('error')
-        self.assertEqual(sorted(error_lines), sorted([
-            'Exception while deleting object %d %d-actual-obj failed to '
-            'delete actual object: ' % (cts, ots),
-            'Exception while deleting object %d %d-next-obj failed to '
-            'delete actual object: ' % (cts, ots),
-            'Exception while deleting object %d %d-actual-obj failed to '
-            'delete actual object: ' % (cts + 1, ots),
-            'Exception while deleting object %d %d-next-obj failed to '
-            'delete actual object: ' % (cts + 1, ots),
-            'Exception while deleting container %d failed to delete '
-            'container: ' % (cts,),
-            'Exception while deleting container %d failed to delete '
-            'container: ' % (cts + 1,)]))
-        self.assertEqual(x.logger.get_lines_for_level('info'), [
-            'Pass beginning; 1 possible containers; 2 possible objects',
+        error_lines = self.expirer.logger.get_lines_for_level('error')
+
+        self.assertEqual(error_lines, [
+            'Exception while deleting object %s %s %s '
+            'failed to delete actual object: ' % (
+                '.expiring_objects', self.past_time,
+                self.past_time + '-' + target_path)
+            for target_path in self.expired_target_path_list] + [
+            'Exception while deleting container %s %s '
+            'failed to delete container: ' % (
+                '.expiring_objects', self.past_time)])
+        self.assertEqual(self.expirer.logger.get_lines_for_level('info'), [
+            'Pass beginning for task account .expiring_objects; '
+            '2 possible containers; 12 possible objects',
             'Pass completed in 0s; 0 objects expired',
         ])
 
@@ -618,7 +765,7 @@ class TestObjectExpirer(TestCase):
         internal_client.loadapp = lambda *a, **kw: fake_app
 
         x = expirer.ObjectExpirer({})
-        ts = '1234'
+        ts = Timestamp('1234')
         x.delete_actual_object('/path/to/object', ts)
         self.assertEqual(got_env[0]['HTTP_X_IF_DELETE_AT'], ts)
         self.assertEqual(got_env[0]['HTTP_X_TIMESTAMP'],
@@ -637,7 +784,7 @@ class TestObjectExpirer(TestCase):
         internal_client.loadapp = lambda *a, **kw: fake_app
 
         x = expirer.ObjectExpirer({})
-        ts = '1234'
+        ts = Timestamp('1234')
         x.delete_actual_object('/path/to/object name', ts)
         self.assertEqual(got_env[0]['HTTP_X_IF_DELETE_AT'], ts)
         self.assertEqual(got_env[0]['HTTP_X_TIMESTAMP'],
@@ -656,11 +803,12 @@ class TestObjectExpirer(TestCase):
             internal_client.loadapp = lambda *a, **kw: fake_app
 
             x = expirer.ObjectExpirer({})
+            ts = Timestamp('1234')
             if should_raise:
                 with self.assertRaises(internal_client.UnexpectedResponse):
-                    x.delete_actual_object('/path/to/object', '1234')
+                    x.delete_actual_object('/path/to/object', ts)
             else:
-                x.delete_actual_object('/path/to/object', '1234')
+                x.delete_actual_object('/path/to/object', ts)
             self.assertEqual(calls[0], 1)
 
         # object was deleted and tombstone reaped
@@ -685,7 +833,7 @@ class TestObjectExpirer(TestCase):
         x = expirer.ObjectExpirer({})
         exc = None
         try:
-            x.delete_actual_object('/path/to/object', '1234')
+            x.delete_actual_object('/path/to/object', Timestamp('1234'))
         except Exception as err:
             exc = err
         finally:
@@ -694,7 +842,7 @@ class TestObjectExpirer(TestCase):
 
     def test_delete_actual_object_quotes(self):
         name = 'this name should get quoted'
-        timestamp = '1366063156.863045'
+        timestamp = Timestamp('1366063156.863045')
         x = expirer.ObjectExpirer({})
         x.swift.make_request = mock.Mock()
         x.swift.make_request.return_value.status_int = 204
@@ -705,7 +853,7 @@ class TestObjectExpirer(TestCase):
 
     def test_delete_actual_object_queue_cleaning(self):
         name = 'something'
-        timestamp = '1515544858.80602'
+        timestamp = Timestamp('1515544858.80602')
         x = expirer.ObjectExpirer({})
         x.swift.make_request = mock.MagicMock()
         x.delete_actual_object(name, timestamp)
@@ -724,13 +872,13 @@ class TestObjectExpirer(TestCase):
             requests.append((method, path))
         with mocked_http_conn(
                 200, 200, 200, give_connect=capture_requests) as fake_conn:
-            x.pop_queue('c', 'o')
+            x.pop_queue('a', 'c', 'o')
             self.assertRaises(StopIteration, fake_conn.code_iter.next)
         for method, path in requests:
             self.assertEqual(method, 'DELETE')
             device, part, account, container, obj = utils.split_path(
                 path, 5, 5, True)
-            self.assertEqual(account, '.expiring_objects')
+            self.assertEqual(account, 'a')
             self.assertEqual(container, 'c')
             self.assertEqual(obj, 'o')
 
